package com.btk5h.skriptdb;

import ch.njol.skript.Skript;
import ch.njol.skript.lang.Expression;
import ch.njol.skript.lang.VariableString;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.Optional;

public class SkriptUtil {
	private static final Field STRING;
	private static final Field EXPR;

	static {
		Field stringField = null;
		Field exprField = null;

		String[] possibleStringFields = {"string", "stringValue", "rawString"};
		for (String fieldName : possibleStringFields) {
			try {
				stringField = VariableString.class.getDeclaredField(fieldName);
				stringField.setAccessible(true);
				break;
			} catch (NoSuchFieldException ignored) {}
		}

		if (stringField == null) {
			for (Field field : VariableString.class.getDeclaredFields()) {
				if (field.getType().isArray() && field.getType().getComponentType() == Object.class) {
					try {
						field.setAccessible(true);
						stringField = field;
						break;
					} catch (SecurityException ignored) {}
				}
			}
		}

		try {
			Optional<Class<?>> expressionInfo = Arrays.stream(VariableString.class.getDeclaredClasses())
					.filter(cls -> cls.getSimpleName().equals("ExpressionInfo"))
					.findFirst();

			if (expressionInfo.isPresent()) {
				Class<?> expressionInfoClass = expressionInfo.get();
				try {
					exprField = expressionInfoClass.getDeclaredField("expr");
					exprField.setAccessible(true);
				} catch (NoSuchFieldException e) {
					for (Field field : expressionInfoClass.getDeclaredFields()) {
						if (Expression.class.isAssignableFrom(field.getType())) {
							field.setAccessible(true);
							exprField = field;
							break;
						}
					}
				}
			}
		} catch (Exception ignored) {}

		STRING = stringField;
		EXPR = exprField;

		if (STRING == null && EXPR == null) {
			Skript.error("Failed to initialize SkriptUtil: Could not find required fields");
		}
	}

	public static Object[] getTemplateString(VariableString vs) {
		try {
			return STRING != null ? (Object[]) STRING.get(vs) : new Object[]{vs.toString()};
		} catch (IllegalAccessException e) {
			return new Object[]{vs.toString()};
		}
	}

	public static Expression<?> getExpressionFromInfo(Object o) {
		try {
			return EXPR != null ? (Expression<?>) EXPR.get(o) : null;
		} catch (IllegalAccessException e) {
			return null;
		}
	}
}